package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.constants.CacheKeyConstants
import com.gxzc.zen.api.sys.mapper.SysDictValueMapper
import com.gxzc.zen.api.sys.model.SysDictValue
import com.gxzc.zen.api.sys.service.ISysDictValueService
import com.gxzc.zen.common.contants.ZenConstants
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.RedisCacheUtil
import org.springframework.stereotype.Service

/**
 * <p>
 * 字典值 服务实现类
 * </p>
 *
 * @author NorthLan
 * @since 2018-04-28
 */
@Suppress("UNCHECKED_CAST")
@Service
class SysDictValueServiceImpl : ServiceImpl<SysDictValueMapper, SysDictValue>(), ISysDictValueService {

    override fun init() {
        getListCacheable()
    }

    override fun getListCacheable(): MutableList<SysDictValue> {
        var data = RedisCacheUtil.get(ZenConstants.CACHE_KEY_SYS, CacheKeyConstants.DICT_VALUE_KEY) as? MutableList<SysDictValue>
        if (data == null || data.isEmpty()) {
            data = baseMapper.selectWOLogic(EntityWrapper<SysDictValue>().orderBy("sort"))
            RedisCacheUtil.put(ZenConstants.CACHE_KEY_SYS, CacheKeyConstants.DICT_VALUE_KEY, data)
        }
        return data
    }

    override fun deleteByTypeId(typeId: Long) {
        val condition = SysDictValue().apply {
            this.typeId = typeId
        }
        baseMapper.physicalDelete(EntityWrapper(condition))
        // 刷新缓存
        evictCache()
    }

    override fun insertDictValue(entity: SysDictValue): SysDictValue {
        if (baseMapper.insert(entity) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        // 刷新缓存
        evictCache()
        return entity
    }

    override fun getListByTypeId(typeId: Long?): MutableList<SysDictValue> {
        if (typeId == null) {
            return mutableListOf()
        }
        val cachedList = getListCacheable()
        return cachedList.filter {
            it.typeId == typeId
        }.toMutableList()
    }

    override fun updateDictValue(entity: SysDictValue): SysDictValue {
        if (entity.id == null) {
            throw ZenException(ZenExceptionEnum.BIZ_UPDATE_ERROR)
        }
        baseMapper.updateWOLogic(entity, EntityWrapper<SysDictValue>().eq("id", entity.id))
        // 刷新缓存
        evictCache()
        return entity
    }

    override fun updateDictValueByTypeId(typeId: Long, oldPrefix: String, newPrefix: String) {
        baseMapper.updateDictValueByTypeId(typeId, oldPrefix, newPrefix)
        evictCache()
    }

    override fun deleteDictValue(id: Long) {
        baseMapper.physicalDeleteById(id)
        // 刷新缓存
        evictCache()
    }

    override fun getDictValueListByTypeCode(typeCode: String): MutableList<SysDictValue> {
        val cachedList = getListCacheable()
        return cachedList.filter {
            it.code!!.startsWith("${typeCode}_")
        }.toMutableList()
    }

    override fun getDictValueByCodeAndValue(code: String, value: String): SysDictValue? {
        val cachedList = getListCacheable()
        return cachedList.find {
            it.code == code && it.value == value
        }
    }

    override fun getDictValueByPrefixAndValue(prefix: String, value: String): SysDictValue? {
        val cachedList = getListCacheable()
        return cachedList.find {
            it.code!!.startsWith("${prefix}_") && it.value == value
        }
    }

    private fun evictCache() {
        RedisCacheUtil.evict(ZenConstants.CACHE_KEY_SYS, CacheKeyConstants.DICT_VALUE_KEY)
    }
}
