package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.dto.ZenFileMetadata
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.util.UploadUtil
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

/**
 * 上传文件 控制器
 * @author NorthLan
 * @date 2018/5/19
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("/upload")
class UploadController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(UploadController::class.java)
    }

    @ApiOperation("获取已上传分片列表")
    @GetMapping
    fun checkChunk(fileMetadata: ZenFileMetadata): ResponseEntity<*> {
        // 检查已上传文件分片
        val ret = UploadUtil.checkUpload(fileMetadata)
        return if (ret.uploadedChunks != null) {
            ResponseEntity.ok(ret)
        } else {
            ResponseEntity.status(204).body(ret)
        }
    }

    @ApiOperation("上传", notes = "支持小文件上传,大文件分片上传(统一分片)")
    @PostMapping
    fun upload(fileMetadata: ZenFileMetadata, file: MultipartFile?): ResponseEntity<*> {
        if (file == null) {
            return ResponseEntity.status(HttpStatus.NOT_ACCEPTABLE.value()).body(null)
        }
        val uploadResponse = try {
            UploadUtil.upload(fileMetadata, file)
        } catch (e: ZenException) {
            null
        }

        return if (uploadResponse != null) {
            ResponseEntity.ok(uploadResponse)
        } else {
            ResponseEntity.status(HttpStatus.NOT_ACCEPTABLE.value()).body(null)
        }
    }
}